package com.trafficparrot.examples.hello.jms;

import org.apache.activemq.ActiveMQConnectionFactory;

import javax.jms.*;
import java.util.concurrent.CountDownLatch;

public class JmsClient {
    public static final String TCP_LOCALHOST_19091 = "tcp://localhost:19091";

    public void sendTextMessageToQueue(String queueName, String text) throws JMSException {
        sendMessageTo(true, queueName, text);
    }

    public String consumeTextMessageFromQueue(String queueName) throws JMSException {
        return consumeTextMessage(queueName);
    }

    public void sendTextMessageToTopic(String topicName, String text) throws JMSException {
        sendMessageTo(false, topicName, text);
    }

    public WaitForOneMessage listenOnTopic(String topic) throws JMSException {
        ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(TCP_LOCALHOST_19091);
        Connection connection = connectionFactory.createConnection();
        return listenOnTopic(topic, connection);
    }

    public WaitForOneMessage listenOnTopic(String topic, Connection connection) throws JMSException {
        connection.setExceptionListener(new PrintStackTraceExceptionListener());
        connection.start();
        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        MessageConsumer consumer = session.createConsumer(session.createTopic(topic));
        WaitForOneMessage listener = new WaitForOneMessage(new CountDownLatch(1), connection, session, consumer);
        consumer.setMessageListener(listener);
        return listener;
    }

    private void sendMessageTo(boolean queue, String name, String text) throws JMSException {
        ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(TCP_LOCALHOST_19091);
        Connection connection = connectionFactory.createConnection();
        connection.setExceptionListener(new PrintStackTraceExceptionListener());
        connection.start();
        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        Destination result;
        if (queue) {
            result = session.createQueue(name);
        } else {
            result = session.createTopic(name);
        }
        Destination destination = result;
        MessageProducer producer = session.createProducer(destination);
        producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
        producer.send(session.createTextMessage(text));
        session.close();
        connection.close();
    }

    private String consumeTextMessage(String queueName) throws JMSException {
        ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(TCP_LOCALHOST_19091);
        Connection connection = connectionFactory.createConnection();
        connection.setExceptionListener(new PrintStackTraceExceptionListener());
        connection.start();
        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        MessageConsumer consumer = session.createConsumer(session.createQueue(queueName));
        Message message = consumer.receive();
        consumer.close();
        session.close();
        connection.close();
        TextMessage textMessage = (TextMessage) message;
        return textMessage.getText();
    }

    private static class PrintStackTraceExceptionListener implements ExceptionListener {

        @Override
        public void onException(JMSException exception) {
            exception.printStackTrace();
        }
    }
}
